#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for PowerShell transcript log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import powershell_transcript

from tests.parsers.text_plugins import test_lib


class PowerShellTranscriptLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for PowerShell transcript log text parser plugin."""

  def testProcess(self):
    """Tests the Process function ."""
    plugin = powershell_transcript.PowerShellTranscriptLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['powershell_transcript.txt'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'build_version': '10.0.17763.1852',
        'clr_version': '4.0.30319.42000',
        'commands': 'PS C:\\Windows\\system32> whoami; msedgewin10\\ieuser',
        'compatible_versions': '1.0, 2.0, 3.0, 4.0, 5.0, 5.1.17763.1852',
        'data_type': 'powershell:transcript_log:entry',
        'edition': 'Desktop',
        'host_application':
            'C:\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe',
        'machine': 'MSEDGEWIN10 (Microsoft Windows NT 10.0.17763.0)',
        'process_identifier': '6456',
        'remoting_protocol_version': '2.3',
        'runas_user': 'MSEDGEWIN10\\IEUser',
        'serialization_version': '1.1.0.1',
        'start_time': '2022-07-21T02:37:49',
        'username': 'MSEDGEWIN10\\IEUser',
        'version': '5.1.17763.1852',
        'ws_man_stack_version': '3.0'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessGerman(self):
    """Tests the Process function on a file with German locale."""
    plugin = powershell_transcript.PowerShellTranscriptLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['powershell_transcript_ger.txt'], plugin)

    # actually 3 Events, but as per l.297 (powershell_transcript.py)
    # currently only the first 2 will be parsed correctly

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'build_version': '10.0.19041.1682',
        'clr_version': '4.0.30319.42000',
        'commands': (
            'Die Aufzeichnung wurde gestartet. '
            'Die Ausgabedatei ist "C:\\Users\\User\\'
            'Documents\\PowerShell_transcript.MySystem.'
            'rvARGulQ.20220824122111.txt".; PS C:\\Users\\User> whoami; '
            'de\\User; PS C:\\Users\\User> echo $null >> filename; '
            'PS C:\\Users\\User> ping 8.8.8.8; Ping wird ausgeführt für '
            '8.8.8.8 mit 32 Bytes Daten:; Zeitüberschreitung der '
            'Anforderung.; Ping-Statistik für 8.8.8.8:; Pakete: '
            'Gesendet = 1, Empfangen = 0, Verloren = 1; (100% Verlust),'
            '; STRG-C; PS C:\\Users\\User> TerminatingError(): "Die '
            'Pipeline wurde beendet."; >> TerminatingError(): "Die '
            'Pipeline wurde beendet."; PS C:\\Users\\User> Get-Content '
            '.\\myfile.txt; VERBOSE: Simple Test'),
        'compatible_versions': '1.0, 2.0, 3.0, 4.0, 5.0, 5.1.19041.1682',
        'data_type': 'powershell:transcript_log:entry',
        'edition': 'Desktop',
        'host_application':
            'C:\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe',
        'machine': 'MySystem (Microsoft Windows NT 10.0.19044.0)',
        'process_identifier': '18716',
        'remoting_protocol_version': '2.3',
        'runas_user': 'DE\\User',
        'serialization_version': '1.1.0.1',
        'start_time': '2022-08-24T12:21:11',
        'username': 'DE\\User',
        'version': '5.1.19041.1682',
        'ws_man_stack_version': '3.0'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'build_version': '10.0.19041.1682',
        'clr_version': '4.0.30319.42000',
        'commands': (
            'PS C:\\Users\\User\\Documents> '
            'ping 127.0.0.1 -n 5; Ping wird ausgeführt '
            'für 127.0.0.1 mit 32 Bytes Daten:; Antwort '
            'von 127.0.0.1: Bytes=32 Zeit<1ms TTL=128; '
            'Antwort von 127.0.0.1: Bytes=32 Zeit<1ms TTL=128; '
            'Antwort von 127.0.0.1: Bytes=32 Zeit<1ms TTL=128; '
            'Antwort von 127.0.0.1: Bytes=32 Zeit=1ms TTL=128; '
            'Antwort von 127.0.0.1: Bytes=32 Zeit<1ms TTL=128; '
            'Ping-Statistik für 127.0.0.1:; Pakete: Gesendet = 5, '
            'Empfangen = 5, Verloren = 0; (0% Verlust),; Ca. '
            'Zeitangaben in Millisek.:; Minimum = 0ms, Maximum '
            '= 1ms, Mittelwert = 0ms'),
        'compatible_versions': '1.0, 2.0, 3.0, 4.0, 5.0, 5.1.19041.1682',
        'data_type': 'powershell:transcript_log:entry',
        'edition': 'Desktop',
        'host_application':
            'C:\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe',
        'machine': 'MySystem (Microsoft Windows NT 10.0.19044.0)',
        'process_identifier': '18716',
        'remoting_protocol_version': '2.3',
        'runas_user': 'DE\\User',
        'serialization_version': '1.1.0.1',
        'start_time': '2022-08-24T12:31:14',
        'username': 'DE\\User',
        'version': '5.1.19041.1682',
        'ws_man_stack_version': '3.0'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Psteal (Plaso SýndarheimsTímalína sem Er ALgjörlega sjálfvirk).

Psteal combines the log2timeline and psort tools into a single tool.
Currently doesn't support any of the two tools flags.

Sample Usage:
  psteal.py --source=/tmp/mystorage.dump --write=/tmp/mystorage_timeline.csv

See additional details here:
  https://plaso.readthedocs.io/en/latest/sources/user/Creating-a-timeline.html#using-psteal
"""

import multiprocessing
import logging
import os
import sys

from plaso import dependencies
from plaso.cli import psteal_tool
from plaso.lib import errors


def Main():
  """Entry point of console script to extract and output events.

  Returns:
    int: exit code that is provided to sys.exit().
  """
  tool = psteal_tool.PstealTool()

  if not tool.ParseArguments(sys.argv[1:]):
    return 1

  if tool.show_troubleshooting:
    print('Using Python version {0!s}'.format(sys.version))
    print()
    print('Path: {0:s}'.format(os.path.abspath(__file__)))
    print()
    print(tool.GetVersionInformation())
    print()
    dependencies.CheckDependencies(verbose_output=True)

    print('Also see: https://plaso.readthedocs.io/en/latest/sources/user/'
          'Troubleshooting.html')
    return 0

  try:
    tool.CheckOutDated()
  except KeyboardInterrupt:
    return 1

  have_list_option = False
  if tool.list_archive_types:
    tool.ListArchiveTypes()
    have_list_option = True

  if tool.list_hashers:
    tool.ListHashers()
    have_list_option = True

  if tool.list_language_tags:
    tool.ListLanguageTags()
    have_list_option = True

  if tool.list_output_modules:
    tool.ListOutputModules()
    have_list_option = True

  if tool.list_parsers_and_plugins:
    tool.ListParsersAndPlugins()
    have_list_option = True

  if tool.list_time_zones:
    tool.ListTimeZones()
    have_list_option = True

  if have_list_option:
    return 0

  if tool.dependencies_check and not dependencies.CheckDependencies(
      verbose_output=False):
    return 1

  try:
    tool.ExtractEventsFromSources()
    tool.ProcessStorage()

  # Writing to stdout and stderr will raise BrokenPipeError if it
  # receives a SIGPIPE.
  except BrokenPipeError:
    pass

  except (KeyboardInterrupt, errors.UserAbort):
    logging.warning('Aborted by user.')
    return 1

  except errors.SourceScannerError as exception:
    logging.warning(exception)
    return 1

  return 0


if __name__ == '__main__':
  multiprocessing.freeze_support()

  sys.exit(Main())



# -*- coding: utf-8 -*-
"""An output module that saves events to OpenSearch."""

from plaso.output import manager
from plaso.output import shared_opensearch


class OpenSearchOutputModule(shared_opensearch.SharedOpenSearchOutputModule):
  """Output module for OpenSearch."""

  NAME = 'opensearch'
  DESCRIPTION = 'Saves the events into an OpenSearch database.'

  MAPPINGS_FILENAME = 'opensearch.mappings'

  def WriteFieldValues(self, output_mediator, field_values):
    """Writes field values to the output.

    Events are buffered in the form of documents and inserted to OpenSearch
    when the flush interval (threshold) has been reached.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      field_values (dict[str, str]): output field values per name.
    """
    event_document = {'index': {'_index': self._index_name}}

    self._event_documents.append(event_document)
    self._event_documents.append(field_values)
    self._number_of_buffered_events += 1

    if self._number_of_buffered_events > self._flush_interval:
      self._FlushEvents()

  def WriteHeader(self, output_mediator):
    """Connects to the OpenSearch server and creates the index.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
    """
    self._Connect()

    self._CreateIndexIfNotExists(self._index_name, self._mappings)


manager.OutputManager.RegisterOutput(
    OpenSearchOutputModule, disabled=shared_opensearch.opensearchpy is None)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Terminal Server Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.parsers.winreg_plugins import terminal_server

from tests.parsers.winreg_plugins import test_lib


class TerminalServerClientPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the Terminal Server Client Connection plugin."""

  def _CreateTestKey(self):
    """Creates Registry keys and values for testing.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString('2012-08-28 09:23:49.002031')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'Servers', key_path_prefix='HKEY_CURRENT_USER',
        last_written_time=filetime.timestamp, offset=865, relative_key_path=(
            'Software\\Microsoft\\Terminal Server Client\\Servers'))

    server_subkey_name = 'myserver.com'
    server_subkey = dfwinreg_fake.FakeWinRegistryKey(
        server_subkey_name, last_written_time=filetime.timestamp, offset=1456)

    registry_key.AddSubkey(server_subkey_name, server_subkey)

    value_data = 'DOMAIN\\username'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'UsernameHint', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=1892)
    server_subkey.AddValue(registry_value)

    return registry_key

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = terminal_server.TerminalServerClientPlugin()

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Terminal Server Client\\Servers'))

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Terminal Server Client\\Default\\AddIns\\RDPDR'))

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', 'Bogus')

  def testProcess(self):
    """Tests the Process function."""
    registry_key = self._CreateTestKey()

    plugin = terminal_server.TerminalServerClientPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Terminal Server Client\\'
        'Servers')

    expected_event_values = {
        'data_type': 'windows:registry:mstsc:connection',
        'key_path': f'{expected_key_path:s}\\myserver.com',
        'last_written_time': '2012-08-28T09:23:49.0020310+00:00',
        'username': 'DOMAIN\\username'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'windows:registry:key_value',
        'key_path': expected_key_path,
        'last_written_time': '2012-08-28T09:23:49.0020310+00:00',
        'values': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


class TerminalServerClientMRUPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the Terminal Server Client Connection MRU plugin."""

  def _CreateTestKey(self):
    """Creates Registry keys and values for testing.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString('2012-08-28 09:23:49.002031')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'Default', key_path_prefix='HKEY_CURRENT_USER',
        last_written_time=filetime.timestamp, offset=1456, relative_key_path=(
            'Software\\Microsoft\\Terminal Server Client\\Default'))

    value_data = '192.168.16.60'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'MRU0', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=1892)
    registry_key.AddValue(registry_value)

    value_data = 'computer.domain.com'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'MRU1', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=612)
    registry_key.AddValue(registry_value)

    return registry_key

  def testProcess(self):
    """Tests the Process function."""
    registry_key = self._CreateTestKey()

    plugin = terminal_server.TerminalServerClientMRUPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Terminal Server Client\\'
        'Default')

    expected_event_values = {
        'data_type': 'windows:registry:mstsc:mru',
        'entries': (
            'MRU0: 192.168.16.60 '
            'MRU1: computer.domain.com'),
        'key_path': expected_key_path,
        'last_written_time': '2012-08-28T09:23:49.0020310+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Firefox cookie database plugins."""

import unittest

from plaso.parsers.sqlite_plugins import firefox_cookies

from tests.parsers.sqlite_plugins import test_lib


class Firefox2CookiesPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Firefox cookie database schema version 2."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = firefox_cookies.FirefoxCookie2Plugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['firefox_cookies', 'firefox_2_cookies.sqlite'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 105)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': '2013-10-30T21:56:03.997686+00:00',
        'cookie_name': '__utma',
        'creation_time': '2013-10-30T21:56:03.992499+00:00',
        'data_type': 'firefox:cookie:entry',
        'expiration_time': '2015-10-30T21:56:03+00:00',
        'host': 's.greenqloud.com',
        'httponly': False,
        'secure': False,
        'url': 'http://s.greenqloud.com/'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 13)
    self.CheckEventData(event_data, expected_event_values)


class Firefox10CookiesPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Firefox cookie database version 10 plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = firefox_cookies.FirefoxCookie10Plugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['firefox_cookies', 'firefox_10_cookies.sqlite'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 13)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': '2023-05-05T16:00:44.811189+00:00',
        'cookie_name': 'WMF-Last-Access',
        'creation_time': '2023-05-05T16:00:44.811189+00:00',
        'data_type': 'firefox:cookie:entry',
        'expiration_time': '2023-06-06T12:00:00+00:00',
        'host': 'en.wikipedia.org',
        'httponly': True,
        'secure': True,
        'url': 'https://en.wikipedia.org/'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 4)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the xchatlog text parser plugin."""

import unittest

from plaso.parsers.text_plugins import xchatlog

from tests.parsers.text_plugins import test_lib


class XChatLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the xchatlog text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = xchatlog.XChatLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['xchat.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': '2011-12-31T21:11:55',
        'data_type': 'xchat:log:line',
        'nickname': None,
        'text': 'XChat start logging'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Interface for compound ZIP file plugins."""

import abc

from plaso.parsers import plugins


class CompoundZIPPlugin(plugins.BasePlugin):
  """Compound ZIP parser plugin."""

  # REQUIRED_PATHS is a list of paths required by a plugin.
  # This is used to understand whether a plugin is suited for a given compound
  # ZIP file.
  # This must be overridden by actual plugins.
  REQUIRED_PATHS = frozenset()

  NAME = 'czip_plugin'
  DATA_FORMAT = 'Compound ZIP file'

  @abc.abstractmethod
  def _ParseZIPFile(self, parser_mediator, zip_file):
    """Extracts events from the ZIP file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      zip_file (zipfile.ZipFile): the ZIP file. It should not be closed in
          this method, but will be closed by the parser logic in czip.py.
    """

  def CheckRequiredPaths(self, zip_file):
    """Checks if the ZIP file has the minimal structure required by the plugin.

    Args:
      zip_file (zipfile.ZipFile): the ZIP file. It should not be closed in
          this method, but will be closed by the parser logic in czip.py.

    Returns:
      bool: True if the ZIP file has the minimum paths defined by the plugin,
          or False if it does not or no required paths are defined. The
          ZIP file can have more paths than specified by the plugin and still
          return True.
    """
    if not self.REQUIRED_PATHS:
      return False

    archive_members = zip_file.namelist()
    return set(self.REQUIRED_PATHS).issubset(archive_members)

  # pylint: disable=arguments-differ
  def Process(self, parser_mediator, zip_file=None, **kwargs):
    """Extracts events from the ZIP file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      zip_file (Optional[zipfile.ZipFile]): the ZIP file. It should not be
          closed in this method, but will be closed by the parser logic in
          czip.py.

    Raises:
      ValueError: If the ZIP file argument is not valid.
    """
    if zip_file is None:
      raise ValueError('Invalid ZIP file.')

    # This will raise if unhandled keyword arguments are passed.
    super(CompoundZIPPlugin, self).Process(parser_mediator)

    self._ParseZIPFile(parser_mediator, zip_file)



# -*- coding: utf-8 -*-
"""This file contains the Task Scheduler Registry keys plugins."""

import os

from dfdatetime import filetime as dfdatetime_filetime

from plaso.containers import events
from plaso.lib import dtfabric_helper
from plaso.lib import errors
from plaso.parsers import winreg_parser
from plaso.parsers.winreg_plugins import interface


class TaskCacheEventData(events.EventData):
  """Task Cache event data.

  Attributes:
    key_path (str): Windows Registry key path.
    last_registered_time (dfdatetime.DateTimeValues): date and time the task
        was last registered.
    last_written_time (dfdatetime.DateTimeValues): entry last written date and
        time.
    launch_time (dfdatetime.DateTimeValues): date and time the task was last
        launched.
    task_name (str): name of the task.
    task_identifier (str): identifier of the task.
    unknown_time (dfdatetime.DateTimeValues): unknown date and time.
  """

  DATA_TYPE = 'task_scheduler:task_cache:entry'

  def __init__(self):
    """Initializes event data."""
    super(TaskCacheEventData, self).__init__(data_type=self.DATA_TYPE)
    self.key_path = None
    self.last_registered_time = None
    self.last_written_time = None
    self.launch_time = None
    self.task_name = None
    self.task_identifier = None
    self.unknown_time = None


class TaskCacheWindowsRegistryPlugin(
    interface.WindowsRegistryPlugin, dtfabric_helper.DtFabricHelper):
  """Plugin that parses a Task Cache key."""

  NAME = 'windows_task_cache'
  DATA_FORMAT = 'Windows Task Scheduler cache Registry data'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\'
          'CurrentVersion\\Schedule\\TaskCache')])

  _DEFINITION_FILE = os.path.join(
      os.path.dirname(__file__), 'task_scheduler.yaml')

  def _GetIdValue(self, registry_key):
    """Retrieves the Id value from Task Cache Tree key.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Yields:
      tuple: containing:

        dfwinreg.WinRegistryKey: Windows Registry key.
        dfwinreg.WinRegistryValue: Windows Registry value.
    """
    id_value = registry_key.GetValueByName('Id')
    if id_value:
      yield registry_key, id_value

    for sub_key in registry_key.GetSubkeys():
      for value_key, id_value in self._GetIdValue(sub_key):
        yield value_key, id_value

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    dynamic_info_size_error_reported = False

    tasks_key = registry_key.GetSubkeyByName('Tasks')
    tree_key = registry_key.GetSubkeyByName('Tree')

    if not tasks_key or not tree_key:
      parser_mediator.ProduceExtractionWarning(
          'Task Cache is missing a Tasks or Tree sub key.')
      return

    task_guids = {}
    for sub_key in tree_key.GetSubkeys():
      for value_key, id_value in self._GetIdValue(sub_key):
        # TODO: improve this check to a regex.
        # The GUID is in the form {%GUID%} and stored an UTF-16 little-endian
        # string and should be 78 bytes in size.
        id_value_data_size = len(id_value.data)
        if id_value_data_size != 78:
          parser_mediator.ProduceExtractionWarning(
              f'unsupported "Id" value data size: {id_value_data_size:d}.')
          continue

        guid_string = id_value.GetDataAsObject()
        task_guids[guid_string] = value_key.name

    dynamic_info_map = self._GetDataTypeMap('dynamic_info_record')
    dynamic_info2_map = self._GetDataTypeMap('dynamic_info2_record')

    for sub_key in tasks_key.GetSubkeys():
      dynamic_info_value = sub_key.GetValueByName('DynamicInfo')
      if not dynamic_info_value:
        continue

      dynamic_info_record_map = None
      dynamic_info_value_data_size = len(dynamic_info_value.data)
      if dynamic_info_value_data_size == 28:
        dynamic_info_record_map = dynamic_info_map
      elif dynamic_info_value_data_size == 36:
        dynamic_info_record_map = dynamic_info2_map
      else:
        if not dynamic_info_size_error_reported:
          parser_mediator.ProduceExtractionWarning((
              f'unsupported "DynamicInfo" value data size: '
              f'{dynamic_info_value_data_size:d}.'))
          dynamic_info_size_error_reported = True
        continue

      try:
        dynamic_info_record = self._ReadStructureFromByteStream(
            dynamic_info_value.data, 0, dynamic_info_record_map)
      except (ValueError, errors.ParseError) as exception:
        dynamic_info_record = None
        parser_mediator.ProduceExtractionWarning(
            f'unable to parse DynamicInfo record with error: {exception!s}.')

      name = task_guids.get(sub_key.name, sub_key.name)

      event_data = TaskCacheEventData()
      event_data.key_path = registry_key.path
      event_data.last_written_time = registry_key.last_written_time
      event_data.task_name = name
      event_data.task_identifier = sub_key.name

      if dynamic_info_record:
        if dynamic_info_record.last_registered_time:
          # Note this is likely either the last registered time or
          # the update time.
          event_data.last_registered_time = dfdatetime_filetime.Filetime(
              timestamp=dynamic_info_record.last_registered_time)

        if dynamic_info_record.launch_time:
          event_data.launch_time = dfdatetime_filetime.Filetime(
              timestamp=dynamic_info_record.launch_time)

        unknown_time = getattr(dynamic_info_record, 'unknown_time', None)
        if unknown_time:
          event_data.unknown_time = dfdatetime_filetime.Filetime(
              timestamp=unknown_time)

      parser_mediator.ProduceEventData(event_data)

    # TODO: Add support for the Triggers value.


winreg_parser.WinRegistryParser.RegisterPlugin(TaskCacheWindowsRegistryPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the CCleaner Windows Registry plugin."""

import unittest

from plaso.parsers.winreg_plugins import ccleaner

from tests.parsers.winreg_plugins import test_lib


class CCleanerRegistryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the CCleaner Windows Registry plugin."""

  # TODO: add tests for _ParseUpdateKeyValue

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = ccleaner.CCleanerPlugin()

    self._AssertFiltersOnKeyPath(
        plugin, 'HKEY_CURRENT_USER\\Software', 'Piriform\\CCleaner')

    self._AssertNotFiltersOnKeyPath(
        plugin, 'HKEY_LOCAL_MACHINE\\Software', 'Bogus')

  def testProcess(self):
    """Tests the Process function."""
    plugin = ccleaner.CCleanerPlugin()
    test_file_entry = self._GetTestFileEntry(['NTUSER-CCLEANER.DAT'])
    key_path = 'HKEY_CURRENT_USER\\Software\\Piriform\\CCleaner'

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'ccleaner:update',
        'key_path': key_path,
        'update_time': '2013-07-13T10:03:14'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_configuration = (
        '(App)Cookies: True '
        '(App)Delete Index.dat files: True '
        '(App)History: True '
        '(App)Last Download Location: True '
        '(App)Other Explorer MRUs: True '
        '(App)Recent Documents: True '
        '(App)Recently Typed URLs: True '
        '(App)Run (in Start Menu): True '
        '(App)Temporary Internet Files: True '
        '(App)Thumbnail Cache: True '
        'CookiesToSave: *.piriform.com '
        'WINDOW_HEIGHT: 524 '
        'WINDOW_LEFT: 146 '
        'WINDOW_MAX: 0 '
        'WINDOW_TOP: 102 '
        'WINDOW_WIDTH: 733')

    expected_event_values = {
        'configuration': expected_configuration,
        'data_type': 'ccleaner:configuration',
        'key_path': key_path,
        'last_written_time': '2013-07-13T14:03:26.8616882+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows XML EventLog (EVTX) parser."""

import unittest

from plaso.parsers import winevtx

from tests.parsers import test_lib


class WinEvtxParserTest(test_lib.ParserTestCase):
  """Tests for the Windows XML EventLog (EVTX) parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = winevtx.WinEvtxParser()
    storage_writer = self._ParseFile(['System.evtx'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1601)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Windows Event Viewer Log (EVTX) information:
    #   Version                     : 3.1
    #   Number of records           : 1601
    #   Number of recovered records : 0
    #   Log type                    : System

    # Event number        : 12049
    # Written time        : Mar 14, 2012 04:17:43.354562700 UTC
    # Event level         : Information (4)
    # Computer name       : WKS-WIN764BITB.shieldbase.local
    # Provider identifier : {fc65ddd8-d6ef-4962-83d5-6e5cfe9ce148}
    # Source name         : Microsoft-Windows-Eventlog
    # Event identifier    : 0x00000069 (105)
    # Number of strings   : 2
    # String: 1           : System
    # String: 2           : C:\Windows\System32\Winevt\Logs\
    #                     : Archive-System-2012-03-14-04-17-39-932.evtx

    expected_string2 = (
        'C:\\Windows\\System32\\Winevt\\Logs\\'
        'Archive-System-2012-03-14-04-17-39-932.evtx')

    expected_xml_string = (
        '<Event xmlns="http://schemas.microsoft.com/win/2004/08/events/'
        'event">\n'
        '  <System>\n'
        '    <Provider Name="Microsoft-Windows-Eventlog" '
        'Guid="{fc65ddd8-d6ef-4962-83d5-6e5cfe9ce148}"/>\n'
        '    <EventID>105</EventID>\n'
        '    <Version>0</Version>\n'
        '    <Level>4</Level>\n'
        '    <Task>105</Task>\n'
        '    <Opcode>0</Opcode>\n'
        '    <Keywords>0x8000000000000000</Keywords>\n'
        '    <TimeCreated SystemTime="2012-03-14T04:17:43.354562700Z"/>\n'
        '    <EventRecordID>12049</EventRecordID>\n'
        '    <Correlation/>\n'
        '    <Execution ProcessID="820" ThreadID="2868"/>\n'
        '    <Channel>System</Channel>\n'
        '    <Computer>WKS-WIN764BITB.shieldbase.local</Computer>\n'
        '    <Security/>\n'
        '  </System>\n'
        '  <UserData>\n'
        '    <AutoBackup '
        'xmlns:auto-ns3="http://schemas.microsoft.com/win/2004/08/events" '
        'xmlns="http://manifests.microsoft.com/win/2004/08/windows/eventlog">\n'
        '      <Channel>System</Channel>\n'
        '      <BackupPath>C:\\Windows\\System32\\Winevt\\Logs\\'
        'Archive-System-2012-03-14-04-17-39-932.evtx</BackupPath>\n'
        '    </AutoBackup>\n'
        '  </UserData>\n'
        '</Event>\n')

    expected_event_values = {
        'creation_time': '2012-03-14T04:17:43.3545627+00:00',
        'computer_name': 'WKS-WIN764BITB.shieldbase.local',
        'data_type': 'windows:evtx:record',
        'event_identifier': 105,
        'event_level': 4,
        'event_version': 0,
        'message_identifier': 105,
        'provider_identifier': '{fc65ddd8-d6ef-4962-83d5-6e5cfe9ce148}',
        'record_number': 12049,
        'source_name': 'Microsoft-Windows-Eventlog',
        'strings': ['System', expected_string2],
        'written_time': '2012-03-14T04:17:43.3545627+00:00',
        'xml_string': expected_xml_string}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testParseTruncated(self):
    """Tests the Parse function on a truncated file."""
    parser = winevtx.WinEvtxParser()
    # Be aware of System2.evtx file, it was manually shortened so it probably
    # contains invalid log at the end.
    storage_writer = self._ParseFile(['System2.evtx'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 194)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:evtx:record',
        'creation_time': '2015-08-09T09:15:48.4341250+00:00',
        'event_identifier': 4624,
        'message_identifier': 4624,
        'record_number': 179,
        'written_time': '2015-08-09T09:15:48.4341250+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 178)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the User Access Logging (UAL) ESE database file."""

import unittest

from plaso.parsers.esedb_plugins import user_access_logging

from tests.parsers.esedb_plugins import test_lib


class UserAccessLoggingESEDBPluginTest(test_lib.ESEDBPluginTestCase):
  """Tests for the User Access Logging (UAL) ESE Database plugin."""

  # pylint: disable=protected-access

  _GUID_BYTES = bytes(bytearray([
      0xc9, 0x8b, 0x91, 0x35, 0x6d, 0x19, 0xea, 0x40, 0x97, 0x79, 0x88, 0x9d,
      0x79, 0xb7, 0x53, 0xf0]))

  def testProcessDatabase(self):
    """Tests processing the database."""
    plugin = user_access_logging.UserAccessLoggingESEDBPlugin()
    storage_writer = self._ParseESEDBFileWithPlugin(
        ['{C519A76A-D9B5-4F85-B667-5FAC08E0E1B4}.mdb'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 25)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:user_access_logging:system_identity',
        'creation_time': '2022-07-16T14:46:12.5700000+00:00',
        'operating_system_build': 17763,
        'system_dns_hostname': 'DC-1',
        'system_domain_name': 'WORKGROUP'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'access_time': '2022-07-16T15:02:52.7424758+00:00',
        'authenticated_username': 'ual\\dc-1$',
        'client_name': None,
        'data_type': 'windows:user_access_logging:clients',
        'insert_time': '2022-07-16T14:50:27.0773450+00:00',
        'role_identifier': '{ad495fc3-0eaa-413d-ba7d-8b13fa7ec598}',
        'role_name': 'Active Directory Domain Services',
        'source_ip_address': '::1',
        'tenant_identifier': '{3facd7dc-85cc-495b-823f-6c96a9e1c40c}',
        'total_accesses': 62}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'windows:user_access_logging:role_access',
        'first_seen_time': '2022-07-16T14:52:50.2088607+00:00',
        'last_seen_time': '2022-07-16T15:00:59.3156655+00:00',
        'role_identifier': '{10a9226f-50ee-49d8-a393-9a501d47ce04}',
        'role_name': 'File Server'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 24)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'windows:user_access_logging:dns',
        'hostname': 'dc-1',
        'ip_address': '10.0.10.10',
        'last_seen_time': '2022-07-16T15:02:53.0830000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 18)
    self.CheckEventData(event_data, expected_event_values)

  def testConvertGUIDToString(self):
    """Tests GUID to string conversion."""
    plugin = user_access_logging.UserAccessLoggingESEDBPlugin()

    guid_string = plugin._ConvertGUIDToString(self._GUID_BYTES)
    self.assertEqual(guid_string, '{35918bc9-196d-40ea-9779-889d79b753f0}')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the default plist plugin."""

import datetime
import unittest

import pytz

from plaso.parsers.plist_plugins import default

from tests.parsers.plist_plugins import test_lib


class TestDefaultPlist(test_lib.PlistPluginTestCase):
  """Tests for the default plist plugin."""

  _TOP_LEVEL_DICT_SINGLE_KEY = {
      'DE-00-AD-00-BE-EF': {
          'Name': 'DBF Industries Slideshow Lazer', 'LastUsed':
          datetime.datetime(
              2012, 11, 2, 1, 21, 38, 997672, tzinfo=pytz.UTC)}}

  _TOP_LEVEL_DICT_MULTIPLE_KEYS = {
      'DeviceCache': {
          '44-00-00-00-00-04': {
              'Name': 'Apple Magic Trackpad 2', 'LMPSubversion': 796,
              'LMPVersion': 3, 'PageScanMode': 0, 'ClassOfDevice': 9620,
              'SupportedFeatures': b'\x00\x00\x00\x00', 'Manufacturer': 76,
              'PageScanPeriod': 0, 'ClockOffset': 17981, 'LastNameUpdate':
              datetime.datetime(
                  2012, 11, 2, 1, 21, 38, 997672, tzinfo=pytz.UTC),
              'InquiryRSSI': 198, 'PageScanRepetitionMode': 1,
              'LastServicesUpdate':
              datetime.datetime(2012, 11, 2, 1, 13, 23, tzinfo=pytz.UTC),
              'displayName': 'Apple Magic Trackpad 2', 'LastInquiryUpdate':
              datetime.datetime(
                  2012, 11, 2, 1, 13, 17, 324095, tzinfo=pytz.UTC),
              'Services': '', 'BatteryPercent': 0.61},
          '44-00-00-00-00-02': {
              'Name': 'test-macpro', 'ClockOffset': 28180,
              'ClassOfDevice': 3670276, 'PageScanMode': 0,
              'LastNameUpdate': datetime.datetime(
                  2011, 4, 7, 17, 56, 53, 524275, tzinfo=pytz.UTC),
              'PageScanPeriod': 2, 'PageScanRepetitionMode': 1,
              'LastInquiryUpdate': datetime.datetime(
                  2012, 7, 10, 22, 5, 0, 20116, tzinfo=pytz.UTC)}}}

  def testProcessSingle(self):
    """Tests Process on a plist containing a root, value and timestamp."""
    plugin = default.DefaultPlugin()
    storage_writer = self._ParsePlistWithPlugin(
        plugin, 'single', self._TOP_LEVEL_DICT_SINGLE_KEY)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'plist:key',
        'key': 'LastUsed',
        'root': '/DE-00-AD-00-BE-EF',
        'written_time': '2012-11-02T01:21:38.997672+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessMulti(self):
    """Tests Process on a plist containing five keys with date values."""
    plugin = default.DefaultPlugin()
    storage_writer = self._ParsePlistWithPlugin(
        plugin, 'nested', self._TOP_LEVEL_DICT_MULTIPLE_KEYS)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 5)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'plist:key',
        'key': 'LastNameUpdate',
        'root': '/DeviceCache/44-00-00-00-00-02',
        'written_time': '2011-04-07T17:56:53.524275+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text plugin for Confluence access log (conf_access_log[DATE].log) files.

Also see:
  https://confluence.atlassian.com/doc/configure-access-logs-1044780567.html
  https://confluence.atlassian.com/confkb/audit-confluence-using-the-tomcat-valve-component-223216846.html
"""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class ConfluenceAccessEventData(events.EventData):
  """Confluence access event data.

  Attributes:
    forwarded_for (str): request X-FORWARDED-FOR header value.
    http_request_method (str): HTTP request method.
    http_request_referer (str): HTTP request referer header information.
    http_request_uri (str): HTTP request URI.
    http_request_user_agent (str): HTTP request user agent header information.
    http_response_bytes (int): HTTP response bytes size without headers.
    http_response_code (int): HTTP response code from server.
    http_version (str): HTTP request version.
    process_duration (int): time taken to process the request in milliseconds.
    recorded_time (dfdatetime.DateTimeValues): date and time the log entry
        was recorded.
    remote_name (str): remote  hostname or IP address
    thread_name (str): name of the thread that handled the request.
    user_name (str): response X-AUSERNAME header value.
  """

  DATA_TYPE = 'confluence:access'

  def __init__(self):
    """Initializes event data."""
    super(ConfluenceAccessEventData, self).__init__(data_type=self.DATA_TYPE)
    self.forwarded_for = None
    self.http_request_method = None
    self.http_request_referer = None
    self.http_request_uri = None
    self.http_request_user_agent = None
    self.http_response_bytes = None
    self.http_response_code = None
    self.http_version = None
    self.process_duration = None
    self.recorded_time = None
    self.remote_name = None
    self.thread_name = None
    self.user_name = None


class ConfluenceAccessTextPlugin(interface.TextPlugin):
  """Text plugin for Confluence access log (conf_access_log[DATE].log) files."""

  NAME = 'confluence_access'
  DATA_FORMAT = 'Confluence access log (access.log) file'

  _INTEGER = pyparsing.Word(pyparsing.nums).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _THREE_LETTERS = pyparsing.Word(pyparsing.alphas, exact=3)

  _MONTH_DICT = {
      'jan': 1,
      'feb': 2,
      'mar': 3,
      'apr': 4,
      'may': 5,
      'jun': 6,
      'jul': 7,
      'aug': 8,
      'sep': 9,
      'oct': 10,
      'nov': 11,
      'dec': 12}

  _TIME_ZONE_OFFSET = (
      pyparsing.Word('+-', exact=1) + _TWO_DIGITS + _TWO_DIGITS)

  # Date and time values are formatted as:
  # [18/Sep/2011:19:18:28 -0400]
  _DATE_TIME = pyparsing.Group(
      pyparsing.Suppress('[') + _TWO_DIGITS +
      pyparsing.Suppress('/') + _THREE_LETTERS +
      pyparsing.Suppress('/') + _FOUR_DIGITS +
      pyparsing.Suppress(':') + _TWO_DIGITS +
      pyparsing.Suppress(':') + _TWO_DIGITS +
      pyparsing.Suppress(':') + _TWO_DIGITS +
      _TIME_ZONE_OFFSET + pyparsing.Suppress(']')).set_results_name('date_time')

  _IP_ADDRESS = (
      pyparsing.pyparsing_common.ipv4_address |
      pyparsing.pyparsing_common.ipv6_address)

  _RESPONSE_BYTES = (
      pyparsing.Literal('-') | _INTEGER).set_results_name('response_bytes')

  _REFERER = pyparsing.Word(pyparsing.alphanums + '/-_.?=%&:+<>#~[]')

  _THREAD_NAME = (
      pyparsing.Word(pyparsing.alphanums + '-').set_results_name('thread_name'))

  _USER_AGENT = pyparsing.restOfLine().set_results_name('user_agent')

  _USER_NAME = (
      pyparsing.Word(pyparsing.alphanums + '@' + pyparsing.alphanums + '.') |
      pyparsing.Word(pyparsing.alphanums) |
      pyparsing.Literal('-')).set_results_name('user_name')

  _HTTP_METHOD = pyparsing.one_of([
      'CONNECT', 'DELETE', 'GET', 'HEAD', 'OPTIONS', 'PATCH', 'POST', 'PUT',
      'TRACE'])

  _REMOTE_NAME = (
      _IP_ADDRESS |
      pyparsing.Word(pyparsing.alphanums + '-' + '.')).set_results_name(
          'remote_name')

  _HTTP_VERSION = pyparsing.Word(pyparsing.alphanums + '/.').set_results_name(
      'http_version')

  _REQUEST_URI = pyparsing.Word(pyparsing.alphanums + '/-_.?=%&:+<>#~[]')

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  # Default (pre 7.11) format log line:
  # %t %{X-AUSERNAME}o %I %h %r %s %Dms %b %{Referer}i %{User-Agent}i

  _PRE_711_FORMAT_LOG_LINE = (
      _DATE_TIME +
      _USER_NAME +
      _THREAD_NAME +
      _REMOTE_NAME +
      _HTTP_METHOD.set_results_name('http_method') +
      _REQUEST_URI.set_results_name('request_url') +
      _HTTP_VERSION +
      _INTEGER.set_results_name('response_code') +
      _INTEGER.set_results_name('process_duration') +
      pyparsing.Literal('ms') +
      _RESPONSE_BYTES +
      _REFERER.set_results_name('referer') +
      _USER_AGENT +
      _END_OF_LINE)

  # Post 7.11 format log line:
  # %t %{X-Forwarded-For}i %{X-AUSERNAME}o %I %h %r %s %Dms %b %{Referer}i
  # %{User-Agent}i

  _POST_711_FORMAT_LOG_LINE = (
      _DATE_TIME +
      _IP_ADDRESS.set_results_name('forwarded_for') +
      _USER_NAME +
      _THREAD_NAME +
      _REMOTE_NAME +
      _HTTP_METHOD.set_results_name('http_method') +
      _REQUEST_URI.set_results_name('request_url') +
      _HTTP_VERSION +
      _INTEGER.set_results_name('response_code') +
      _INTEGER.set_results_name('process_duration') +
      pyparsing.Literal('ms') +
      _RESPONSE_BYTES +
      _REFERER.set_results_name('referer') +
      _USER_AGENT +
      _END_OF_LINE)

  _LINE_STRUCTURES = [
      ('pre_711_format', _PRE_711_FORMAT_LOG_LINE),
      ('post_711_format', _POST_711_FORMAT_LOG_LINE)]

  VERIFICATION_GRAMMAR = _PRE_711_FORMAT_LOG_LINE ^ _POST_711_FORMAT_LOG_LINE

  VERIFICATION_LITERALS = [
      ' CONNECT ', ' DELETE ', ' GET ', ' HEAD ', ' HTTP/', ' OPTIONS ',
      ' PATCH ', ' POST ', ' PUT ', ' TRACE ']

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    event_data = ConfluenceAccessEventData()
    event_data.http_request_method = self._GetValueFromStructure(
        structure, 'http_method')
    event_data.http_request_referer = self._GetValueFromStructure(
        structure, 'referer')
    event_data.http_request_uri = self._GetValueFromStructure(
        structure, 'request_url')
    event_data.http_request_user_agent = self._GetStringValueFromStructure(
        structure, 'user_agent')
    event_data.http_response_code = self._GetValueFromStructure(
        structure, 'response_code')
    event_data.http_response_bytes = self._GetValueFromStructure(
        structure, 'response_bytes')
    event_data.http_version = self._GetValueFromStructure(
        structure, 'http_version')
    event_data.process_duration = self._GetValueFromStructure(
        structure, 'process_duration')
    event_data.recorded_time = self._ParseTimeElements(time_elements_structure)
    event_data.remote_name = self._GetValueFromStructure(
        structure, 'remote_name')
    event_data.thread_name = self._GetValueFromStructure(
        structure, 'thread_name')
    event_data.user_name = self._GetValueFromStructure(
        structure, 'user_name')

    if key == 'post_711_format':
      event_data.forwarded_for = self._GetValueFromStructure(
          structure, 'forwarded_for')

    parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      (day_of_month, month_string, year, hours, minutes, seconds,
       time_zone_sign, time_zone_hours, time_zone_minutes) = (
          time_elements_structure)

      month = self._MONTH_DICT.get(month_string.lower(), 0)

      time_zone_offset = (time_zone_hours * 60) + time_zone_minutes
      if time_zone_sign == '-':
        time_zone_offset *= -1

      time_elements_tuple = (year, month, day_of_month, hours, minutes, seconds)

      return dfdatetime_time_elements.TimeElements(
          time_elements_tuple=time_elements_tuple,
          time_zone_offset=time_zone_offset)

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          f'Unable to parse time elements with error: {exception!s}')

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    try:
      self._ParseTimeElements(time_elements_structure)
    except errors.ParseError:
      return False

    return True


text_parser.TextLogParser.RegisterPlugin(ConfluenceAccessTextPlugin)



# -*- coding: utf-8 -*-
"""Text plugin for MacOS Application firewall log (appfirewall.log) files."""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import dateless_helper
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class MacOSAppFirewallLogEventData(events.EventData):
  """MacOS Application firewall log (appfirewall.log) file event data.

  Attributes:
    action (str): action.
    added_time (dfdatetime.DateTimeValues): date and time the log entry
        was added.
    agent (str): agent that save the log.
    computer_name (str): name of the computer.
    process_name (str): name of the entity that tried to do the action.
    status (str): saved status action.
  """

  DATA_TYPE = 'macos:appfirewall_log:entry'

  def __init__(self):
    """Initializes event data."""
    super(MacOSAppFirewallLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.action = None
    self.added_time = None
    self.agent = None
    self.computer_name = None
    self.process_name = None
    self.status = None


class MacOSAppFirewallTextPlugin(
    interface.TextPlugin, dateless_helper.DateLessLogFormatHelper):
  """Text plugin for MacOS Application firewall log (appfirewall.log) files."""

  NAME = 'mac_appfirewall_log'
  DATA_FORMAT = 'MacOS Application firewall log (appfirewall.log) file'

  ENCODING = 'utf-8'

  _ONE_OR_TWO_DIGITS = pyparsing.Word(pyparsing.nums, max=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _THREE_LETTERS = pyparsing.Word(pyparsing.alphas, exact=3)

  # Define how a log line should look like.
  # Example: 'Nov  2 04:07:35 DarkTemplar-2.local socketfilterfw[112] '
  #          '<Info>: Dropbox: Allow (in:0 out:2)'
  # INFO: process_name is going to have a white space at the beginning.

  _DATE_TIME = (
      _THREE_LETTERS +
      _ONE_OR_TWO_DIGITS +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress(':') + _TWO_DIGITS)

  _REPORTER_CHARACTERS = ''.join(
      [c for c in pyparsing.printables if c not in [':', '[', '<']])

  _PROCESS_IDENTIFIER = pyparsing.Word(pyparsing.nums, max=5).set_parse_action(
      lambda tokens: int(tokens[0], 10))

  _AGENT = pyparsing.Combine(
      pyparsing.Word(_REPORTER_CHARACTERS) + pyparsing.Literal('[') +
      _PROCESS_IDENTIFIER + pyparsing.Literal(']'))

  _LOG_LINE_START = (
      _DATE_TIME.set_results_name('date_time') +
      pyparsing.Word(pyparsing.printables).set_results_name('computer_name') +
      _AGENT.set_results_name('agent'))

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  _LOG_LINE = (
      _LOG_LINE_START +
      pyparsing.Suppress('<') +
      pyparsing.CharsNotIn('>').set_results_name('status') +
      pyparsing.Suppress('>:') +
      pyparsing.CharsNotIn(':').set_results_name('process_name') +
      pyparsing.Suppress(': ') +
      pyparsing.restOfLine().set_results_name('action') + _END_OF_LINE)

  # Repeated line.
  # Example: Nov 29 22:18:29 --- last message repeated 1 time ---

  _REPEATED_LOG_LINE_END = pyparsing.Suppress('---') + _END_OF_LINE

  _REPEATED_LOG_LINE = (
      _DATE_TIME.set_results_name('date_time') +
      pyparsing.Suppress('---') + pyparsing.SkipTo(_REPEATED_LOG_LINE_END) +
      _REPEATED_LOG_LINE_END)

  _LINE_STRUCTURES = [
      ('log_line', _LOG_LINE),
      ('repeated_log_line', _REPEATED_LOG_LINE)]

  VERIFICATION_GRAMMAR = _LOG_LINE_START + pyparsing.Literal(
      '<Error>: Logging: creating /var/log/appfirewall.log') + _END_OF_LINE

  def __init__(self):
    """Initializes a text parser plugin."""
    super(MacOSAppFirewallTextPlugin, self).__init__()
    self._repeated_structure = None

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    # If the actual entry is a repeated entry, we take the basic information
    # from the previous entry, but use the timestamp from the actual entry.
    if key == 'log_line':
      self._repeated_structure = structure
    else:
      structure = self._repeated_structure

    event_data = MacOSAppFirewallLogEventData()
    event_data.action = self._GetValueFromStructure(structure, 'action')
    event_data.added_time = self._ParseTimeElements(time_elements_structure)
    event_data.agent = self._GetValueFromStructure(structure, 'agent')
    event_data.computer_name = self._GetValueFromStructure(
        structure, 'computer_name')
    event_data.process_name = self._GetStringValueFromStructure(
        structure, 'process_name')
    event_data.status = self._GetValueFromStructure(structure, 'status')

    parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      month_string, day_of_month, hours, minutes, seconds = (
          time_elements_structure)

      month = self._GetMonthFromString(month_string)

      self._UpdateYear(month)

      relative_year = self._GetRelativeYear()

      time_elements_tuple = (
          relative_year, month, day_of_month, hours, minutes, seconds)

      return dfdatetime_time_elements.TimeElements(
          is_delta=True, time_elements_tuple=time_elements_tuple)

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct plugin, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    self._SetEstimatedYear(parser_mediator)

    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    try:
      self._ParseTimeElements(time_elements_structure)
    except errors.ParseError:
      return False

    self._repeated_structure = None

    return True


text_parser.TextLogParser.RegisterPlugin(MacOSAppFirewallTextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the text plugins interface."""

import unittest

from tests.parsers import test_lib


class TextPluginTest(test_lib.ParserTestCase):
  """Tests for the text plugins interface."""

  # pylint: disable=protected-access

  # TODO: add tests for _GetMatchingLineStructure
  # TODO: add tests for _GetValueFromStructure
  # TODO: add tests for _ParseLines
  # TODO: add tests for _ParseLineStructure
  # TODO: add tests for _SetLineStructures
  # TODO: add tests for Process


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MSIE zone settings Windows Registry plugin."""

import unittest

from plaso.parsers.winreg_plugins import msie_zones

from tests.parsers.winreg_plugins import test_lib


class MSIEZoneSettingsPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for Internet Settings zone settings plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = msie_zones.MSIEZoneSettingsPlugin()

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings\\'
        'Lockdown_Zones'))

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings\\'
        'Zones'))

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_CURRENT_USER', (
        'Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings\\'
        'Lockdown_Zones'))

    self._AssertFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Software', (
        'Microsoft\\Windows\\CurrentVersion\\Internet Settings\\Zones'))

    self._AssertNotFiltersOnKeyPath(
        plugin, 'HKEY_LOCAL_MACHINE\\Software', 'Bogus')

  def testProcessNtuserLockdownZones(self):
    """Tests the Process function on a Lockdown_Zones key."""
    test_file_entry = self._GetTestFileEntry(['NTUSER-WIN7.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Internet Settings\\Lockdown_Zones')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = msie_zones.MSIEZoneSettingsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_settings = (
        '[1200] Run ActiveX controls and plug-ins: 3 (Not Allowed) '
        '[1400] Active scripting: 1 (Prompt User) '
        '[CurrentLevel]: 0 '
        '[Description]: Your computer '
        '[DisplayName]: Computer '
        '[Flags]: 33 '
        '[Icon]: shell32.dll#0016 '
        '[LowIcon]: inetcpl.cpl#005422 '
        '[PMDisplayName]: Computer '
        '[Protected Mode]')

    expected_event_values = {
        'data_type': 'windows:registry:msie_zone_settings',
        'key_path': f'{key_path:s}\\0 (My Computer)',
        'last_written_time': '2011-09-16T21:12:40.1455141+00:00',
        'settings': expected_settings}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessNtuserZones(self):
    """Tests the Process function on a Zones key."""
    test_file_entry = self._GetTestFileEntry(['NTUSER-WIN7.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Internet Settings\\Zones')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = msie_zones.MSIEZoneSettingsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_settings = (
        '[1200] Run ActiveX controls and plug-ins: 0 (Allow) '
        '[1400] Active scripting: 0 (Allow) '
        '[2001] .NET: Run components signed with Authenticode: 3 (Not '
        'Allowed) '
        '[2004] .NET: Run components not signed with Authenticode: 3 (Not '
        'Allowed) '
        '[2007] UNKNOWN: 3 '
        '[CurrentLevel]: 0 '
        '[Description]: Your computer '
        '[DisplayName]: Computer '
        '[Flags]: 33 [Icon]: shell32.dll#0016 '
        '[LowIcon]: inetcpl.cpl#005422 '
        '[PMDisplayName]: Computer '
        '[Protected Mode]')

    expected_event_values = {
        'data_type': 'windows:registry:msie_zone_settings',
        'key_path': f'{key_path:s}\\0 (My Computer)',
        'last_written_time': '2011-09-16T21:12:40.1455141+00:00',
        'settings': expected_settings}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessSoftwareLockdownZones(self):
    """Tests the Process function on a Lockdown_Zones key."""
    test_file_entry = self._GetTestFileEntry(['SOFTWARE'])
    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Internet Settings\\Lockdown_Zones')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = msie_zones.MSIEZoneSettingsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_settings = (
        '[1001] Download signed ActiveX controls: 1 (Prompt User) '
        '[1004] Download unsigned ActiveX controls: 3 (Not Allowed) '
        '[1200] Run ActiveX controls and plug-ins: 3 (Not Allowed) '
        '[1201] Initialize and script ActiveX controls not marked as safe: 3 '
        '(Not Allowed) '
        '[1206] Allow scripting of IE Web browser control: 0 '
        '[1207] Reserved: 3 '
        '[1208] Allow previously unused ActiveX controls to run without '
        'prompt: 3 '
        '[1209] Allow Scriptlets: 3 '
        '[120A] Override Per-Site (domain-based) ActiveX restrictions: 3 '
        '[120B] Override Per-Site (domain-based) ActiveX restrictions: 0 '
        '[1400] Active scripting: 1 (Prompt User) '
        '[1402] Scripting of Java applets: 0 (Allow) '
        '[1405] Script ActiveX controls marked as safe for scripting: 0 '
        '(Allow) '
        '[1406] Access data sources across domains: 0 (Allow) '
        '[1407] Allow Programmatic clipboard access: 1 (Prompt User) '
        '[1408] Reserved: 3 '
        '[1409] UNKNOWN: 3 '
        '[1601] Submit non-encrypted form data: 0 (Allow) '
        '[1604] Font download: 0 (Allow) '
        '[1605] Run Java: 0 '
        '[1606] Userdata persistence: 0 (Allow) '
        '[1607] Navigate sub-frames across different domains: 0 (Allow) '
        '[1608] Allow META REFRESH: 0 (Allow) '
        '[1609] Display mixed content: 1 (Prompt User) '
        '[160A] Include local directory path when uploading files to a '
        'server: 0 '
        '[1802] Drag and drop or copy and paste files: 0 (Allow) '
        '[1803] File Download: 0 (Allow) '
        '[1804] Launching programs and files in an IFRAME: 0 (Allow) '
        '[1805] Launching programs and files in webview: 0 '
        '[1806] Launching applications and unsafe files: 0 '
        '[1807] Reserved: 0 '
        '[1808] Reserved: 0 '
        '[1809] Use Pop-up Blocker: 3 (Not Allowed) '
        '[180A] Reserved: 0 '
        '[180C] Reserved: 0 '
        '[180D] Reserved: 0 '
        '[180E] UNKNOWN: 0 '
        '[180F] UNKNOWN: 0 '
        '[1A00] User Authentication: Logon: 0x00000000 (Automatic logon with '
        'current user name and password) '
        '[1A02] Allow persistent cookies that are stored on your computer: 0 '
        '[1A03] Allow per-session cookies (not stored): 0 '
        '[1A04] Don\'t prompt for client cert selection when no certs exists: '
        '3 (Not Allowed) '
        '[1A05] Allow 3rd party persistent cookies: 0 '
        '[1A06] Allow 3rd party session cookies: 0 '
        '[1A10] Privacy Settings: 0 '
        '[1C00] Java permissions: 0x00000000 (Disable Java) '
        '[2000] Binary and script behaviors: 0x00010000 '
        '(Administrator approved) '
        '[2005] UNKNOWN: 3 '
        '[2100] Open files based on content, not file extension: 3 '
        '(Not Allowed) '
        '[2101] Web sites in less privileged zone can navigate into this '
        'zone: 3 (Not Allowed) '
        '[2102] Allow script initiated windows without size/position '
        'constraints: '
        '3 (Not Allowed) '
        '[2103] Allow status bar updates via script: 3 '
        '[2104] Allow websites to open windows without address or status '
        'bars: 3 '
        '[2105] Allow websites to prompt for information using scripted '
        'windows: 3 '
        '[2106] UNKNOWN: 3 '
        '[2107] UNKNOWN: 3 '
        '[2200] Automatic prompting for file downloads: 3 (Not Allowed) '
        '[2201] Automatic prompting for ActiveX controls: 3 (Not Allowed) '
        '[2301] Use Phishing Filter: 3 '
        '[2400] .NET: XAML browser applications: 0 '
        '[2401] .NET: XPS documents: 0 '
        '[2402] .NET: Loose XAML: 0 '
        '[2500] Turn on Protected Mode: 3 '
        '[2600] Enable .NET Framework setup: 0 '
        '[2700] UNKNOWN: 3 '
        '[2701] UNKNOWN: 3 '
        '[2702] UNKNOWN: 3 '
        '[2703] UNKNOWN: 3 '
        '[2708] UNKNOWN: 0 '
        '[2709] UNKNOWN: 0 '
        '[CurrentLevel]: 0 '
        '[Description]: Your computer '
        '[DisplayName]: Computer '
        '[Flags]: 33 '
        '[Icon]: shell32.dll#0016 '
        '[LowIcon]: inetcpl.cpl#005422 '
        '[PMDisplayName]: Computer '
        '[Protected Mode]')

    expected_event_values = {
        'data_type': 'windows:registry:msie_zone_settings',
        'key_path': f'{key_path:s}\\0 (My Computer)',
        'last_written_time': '2011-08-28T21:32:44.9376751+00:00',
        'settings': expected_settings}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessSoftwareZones(self):
    """Tests the Process function on a Zones key."""
    test_file_entry = self._GetTestFileEntry(['SOFTWARE'])
    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Internet Settings\\Zones')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = msie_zones.MSIEZoneSettingsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_settings = (
        '[1001] Download signed ActiveX controls: 0 (Allow) '
        '[1004] Download unsigned ActiveX controls: 0 (Allow) '
        '[1200] Run ActiveX controls and plug-ins: 0 (Allow) '
        '[1201] Initialize and script ActiveX controls not marked as safe: 1 '
        '(Prompt User) '
        '[1206] Allow scripting of IE Web browser control: 0 '
        '[1207] Reserved: 0 '
        '[1208] Allow previously unused ActiveX controls to run without '
        'prompt: 0 '
        '[1209] Allow Scriptlets: 0 '
        '[120A] Override Per-Site (domain-based) ActiveX restrictions: 0 '
        '[120B] Override Per-Site (domain-based) ActiveX restrictions: 0 '
        '[1400] Active scripting: 0 (Allow) '
        '[1402] Scripting of Java applets: 0 (Allow) '
        '[1405] Script ActiveX controls marked as safe for scripting: 0 '
        '(Allow) '
        '[1406] Access data sources across domains: 0 (Allow) '
        '[1407] Allow Programmatic clipboard access: 0 (Allow) '
        '[1408] Reserved: 0 '
        '[1409] UNKNOWN: 3 '
        '[1601] Submit non-encrypted form data: 0 (Allow) '
        '[1604] Font download: 0 (Allow) '
        '[1605] Run Java: 0 '
        '[1606] Userdata persistence: 0 (Allow) '
        '[1607] Navigate sub-frames across different domains: 0 (Allow) '
        '[1608] Allow META REFRESH: 0 (Allow) '
        '[1609] Display mixed content: 1 (Prompt User) '
        '[160A] Include local directory path when uploading files to a '
        'server: 0 '
        '[1802] Drag and drop or copy and paste files: 0 (Allow) '
        '[1803] File Download: 0 (Allow) '
        '[1804] Launching programs and files in an IFRAME: 0 (Allow) '
        '[1805] Launching programs and files in webview: 0 '
        '[1806] Launching applications and unsafe files: 0 '
        '[1807] Reserved: 0 '
        '[1808] Reserved: 0 '
        '[1809] Use Pop-up Blocker: 3 (Not Allowed) '
        '[180A] Reserved: 0 '
        '[180C] Reserved: 0 '
        '[180D] Reserved: 0 '
        '[180E] UNKNOWN: 0 '
        '[180F] UNKNOWN: 0 '
        '[1A00] User Authentication: Logon: 0x00000000 (Automatic logon with '
        'current user name and password) '
        '[1A02] Allow persistent cookies that are stored on your computer: 0 '
        '[1A03] Allow per-session cookies (not stored): 0 '
        '[1A04] Don\'t prompt for client cert selection when no certs exists: '
        '0 (Allow) '
        '[1A05] Allow 3rd party persistent cookies: 0 '
        '[1A06] Allow 3rd party session cookies: 0 '
        '[1A10] Privacy Settings: 0 '
        '[1C00] Java permissions: 0x00020000 (Medium safety) '
        '[2000] Binary and script behaviors: 0 (Allow) '
        '[2001] .NET: Run components signed with Authenticode: '
        '3 (Not Allowed) '
        '[2004] .NET: Run components not signed with Authenticode: '
        '3 (Not Allowed) '
        '[2005] UNKNOWN: 0 '
        '[2007] UNKNOWN: 3 '
        '[2100] Open files based on content, not file extension: 0 (Allow) '
        '[2101] Web sites in less privileged zone can navigate into this '
        'zone: 3 (Not Allowed) '
        '[2102] Allow script initiated windows without size/position '
        'constraints: 0 (Allow) '
        '[2103] Allow status bar updates via script: 0 '
        '[2104] Allow websites to open windows without address or status '
        'bars: 0 '
        '[2105] Allow websites to prompt for information using scripted '
        'windows: 0 '
        '[2106] UNKNOWN: 0 '
        '[2107] UNKNOWN: 0 '
        '[2200] Automatic prompting for file downloads: 0 (Allow) '
        '[2201] Automatic prompting for ActiveX controls: 0 (Allow) '
        '[2300] Allow web pages to use restricted protocols for active '
        'content: 1 (Prompt User) '
        '[2301] Use Phishing Filter: 3 '
        '[2400] .NET: XAML browser applications: 0 '
        '[2401] .NET: XPS documents: 0 '
        '[2402] .NET: Loose XAML: 0 '
        '[2500] Turn on Protected Mode: 3 '
        '[2600] Enable .NET Framework setup: 0 '
        '[2700] UNKNOWN: 3 '
        '[2701] UNKNOWN: 0 '
        '[2702] UNKNOWN: 3 '
        '[2703] UNKNOWN: 3 '
        '[2708] UNKNOWN: 0 '
        '[2709] UNKNOWN: 0 '
        '[CurrentLevel]: 0 '
        '[Description]: Your computer '
        '[DisplayName]: Computer '
        '[Flags]: 33 '
        '[Icon]: shell32.dll#0016 '
        '[LowIcon]: inetcpl.cpl#005422 '
        '[PMDisplayName]: Computer '
        '[Protected Mode]')

    expected_event_values = {
        'data_type': 'windows:registry:msie_zone_settings',
        'key_path': f'{key_path:s}\\0 (My Computer)',
        'last_written_time': '2011-08-28T21:32:44.9376751+00:00',
        'settings': expected_settings}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the AWS ELB access log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import aws_elb_access

from tests.parsers.text_plugins import test_lib


class AWSELBTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the AWS ELB access log text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = aws_elb_access.AWSELBTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['aws_elb_access.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 16)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'actions_executed': 'waf,forward',
        'chosen_cert_arn': 'arn:aws:abc:us-east-2:234567891234:certificate',
        'classification': None,
        'classification_reason': None,
        'data_type': 'aws:elb:access',
        'destination_group_arn': (
            'arn:aws:elasticloadbalancing:us-east-2:123456789123'),
        'destination_ip_address': '192.168.1.123',
        'destination_list': ['192.168.1.123:32869'],
        'destination_port': 32869,
        'destination_processing_duration': '0.164',
        'destination_status_code': 200,
        'destination_status_code_list': '200',
        'domain_name': 'www.domain.name',
        'elb_status_code': 200,
        'error_reason': None,
        'matched_rule_priority': 2,
        'received_bytes': 391,
        'redirect_url': None,
        'request': 'GET https://www.domain.name:443/ HTTP/1.1',
        'request_processing_duration': '0.013',
        'request_type': 'https',
        'resource_identifier': 'app/production-web/jf29fj2198ejf922',
        'response_processing_duration': '0.000',
        'sent_bytes': 107999,
        'source_ip_address': '192.168.1.10',
        'source_port': 44325,
        'ssl_cipher': 'ECDHE-RSA-AES128-GCM-SHA256',
        'ssl_protocol': 'TLSv1.2',
        'request_time': '2020-01-11T16:55:19.624000+00:00',
        'response_time': '2020-01-11T16:55:20.356586+00:00',
        'trace_identifier': 'XXXXXXX',
        'user_agent': (
            'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; '
            'Trident/6.0)')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Android turbo plugin."""

import unittest

from plaso.parsers.sqlite_plugins import android_turbo

from tests.parsers.sqlite_plugins import test_lib


class AndroidTurboSQLitePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Android turbo database plugin."""

  def testProcess(self):
    """Tests the Process function on an Android turbo.db file."""
    plugin = android_turbo.AndroidTurboPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['android_turbo.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2717)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'battery_level': 99,
        'battery_saver': 2,
       'charge_type': 0,
       'recorded_time': '2023-05-27T13:06:46.000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'battery_level': 54,
        'battery_saver': 2,
        'charge_type': 1,
        'recorded_time': '2023-06-22T11:26:27.000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2138)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text file parser plugin for Container Runtime Interface (CRI) log format.

This is a text-based log format used in kubernetes/GKE.

Also see:
  https://github.com/kubernetes/design-proposals-archive/blob/main/node/kubelet-cri-logging.md
"""

import pyparsing

from dfdatetime import time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class CRIEventData(events.EventData):
  """CRI log event data.

  Attributes:
    body (str): the log message body.
    event_datetime (time_elements.TimeElementsInNanoseconds): the datetime of
        the log message.
    stream (str): the log stream.  Currently only 'stdout' and 'stderr' are
        supported.
    tag (str): the log tag.  Currently only 'P' (partial) and 'F' (full) are
        supported.
  """
  DATA_TYPE = 'cri:container:log:entry'

  def __init__(self):
    """Initializes event data."""
    super(CRIEventData, self).__init__(data_type=self.DATA_TYPE)
    self.body = None
    self.event_datetime = None
    self.stream = None
    self.tag = None


class CRITextPlugin(interface.TextPlugin):
  """Text file parser plugin for CRI log files."""

  NAME = 'cri_log'
  DATA_FORMAT = 'Container Runtime Interface log file'

  ENCODING = 'utf-8'

  # Date and time values are formatted as: 2016-10-06T00:17:09.669794202Z
  _DATE_AND_TIME = (
      pyparsing.Regex(r'\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}.\d{1,9}Z')
  ).setResultsName('date_time')

  _STREAM = (
      pyparsing.Literal('stderr') ^ pyparsing.Literal('stdout')
  ).setResultsName('stream')

  # P indicates a partial log,
  # F indicates a complete or the end of a multiline log.
  _TAG = pyparsing.oneOf(['P', 'F']).setResultsName('tag')

  _LOG = (
      pyparsing.restOfLine() + pyparsing.Suppress(pyparsing.LineEnd())
  ).setResultsName('body')

  _LOG_LINE = _DATE_AND_TIME + _STREAM + _TAG + _LOG
  _LINE_STRUCTURES = [('log_line', _LOG_LINE)]

  VERIFICATION_GRAMMAR = _LOG_LINE

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    if key == 'log_line':
      date_time = time_elements.TimeElementsInNanoseconds()
      date_time.CopyFromStringISO8601(self._GetValueFromStructure(
          structure, 'date_time'))
      event_data = CRIEventData()
      event_data.event_datetime = date_time
      event_data.body = self._GetValueFromStructure(
          structure, 'body')[0]
      event_data.stream = self._GetValueFromStructure(structure, 'stream')
      event_data.tag = self._GetValueFromStructure(structure, 'tag')
      parser_mediator.ProduceEventData(event_data)

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the parser.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    try:
      self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    return True


text_parser.TextLogParser.RegisterPlugin(CRITextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Google Chrome History database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import chrome_history

from tests.parsers.sqlite_plugins import test_lib


class GoogleChrome8HistoryPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Chrome 8 history SQLite database plugin."""

  def testProcess(self):
    """Tests the Process function on a Chrome History database file."""
    plugin = chrome_history.GoogleChrome8HistoryPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['chrome_history', 'History'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 71)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Check the first page visited entry.
    expected_event_values = {
        'data_type': 'chrome:history:page_visited',
        'last_visited_time': '2011-04-07T12:03:11.000000+00:00',
        'page_transition_type': 0,
        'title': 'Ubuntu Start Page',
        'typed_count': 0,
        'url': 'http://start.ubuntu.com/10.04/Google/',
        'visit_count': 4,
        'visit_source': 3}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Check the first file downloaded entry.
    expected_event_values = {
        'data_type': 'chrome:history:file_downloaded',
        'end_time': None,
        'full_path': '/home/john/Downloads/funcats_scr.exe',
        'received_bytes': 1132155,
        'start_time': '2011-05-23T08:35:30+00:00',
        'total_bytes': 1132155,
        'url': 'http://fatloss4idiotsx.com/download/funcats/funcats_scr.exe'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 69)
    self.CheckEventData(event_data, expected_event_values)


class GoogleChrome27HistoryPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Chrome 27 history SQLite database plugin."""

  def testProcess57(self):
    """Tests the Process function on a Google Chrome 57 History database."""
    plugin = chrome_history.GoogleChrome27HistoryPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['chrome_history', 'History-57.0.2987.133'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Check the page visit entry.
    expected_url = (
        'https://raw.githubusercontent.com/dfirlabs/chrome-specimens/master/'
        'generate-specimens.sh')

    expected_event_values = {
        'data_type': 'chrome:history:page_visited',
        'last_visited_time': '2018-01-21T14:09:53.885478+00:00',
        'title': '',
        'typed_count': 0,
        'visit_count': 1,
        'url': expected_url}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Check the file downloaded entry.
    expected_event_values = {
        'data_type': 'chrome:history:file_downloaded',
        'end_time': '2018-01-21T14:09:54.858738+00:00',
        'full_path': '/home/ubuntu/Downloads/plaso-20171231.1.win32.msi',
        'received_bytes': 3080192,
        'start_time': '2018-01-21T14:09:53.900399+00:00',
        'total_bytes': 3080192,
        'url': (
            'https://raw.githubusercontent.com/log2timeline/l2tbinaries/master/'
            'win32/plaso-20171231.1.win32.msi')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

  def testProcess58(self):
    """Tests the Process function on a Google Chrome 58 History database."""
    plugin = chrome_history.GoogleChrome27HistoryPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['chrome_history', 'History-58.0.3029.96'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Check the page visit entry.
    expected_url = (
        'https://raw.githubusercontent.com/dfirlabs/chrome-specimens/master/'
        'generate-specimens.sh')

    expected_event_values = {
        'data_type': 'chrome:history:page_visited',
        'last_visited_time': '2018-01-21T14:09:27.315765+00:00',
        'title': '',
        'typed_count': 0,
        'visit_count': 1,
        'url': expected_url}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Check the file downloaded entry.
    expected_event_values = {
        'data_type': 'chrome:history:file_downloaded',
        'end_time': '2018-01-21T14:09:28.116062+00:00',
        'full_path': '/home/ubuntu/Downloads/plaso-20171231.1.win32.msi',
        'received_bytes': 3080192,
        'start_time': '2018-01-21T14:09:27.200398+00:00',
        'total_bytes': 3080192,
        'url': (
            'https://raw.githubusercontent.com/log2timeline/l2tbinaries/master/'
            'win32/plaso-20171231.1.win32.msi')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

  def testProcess59(self):
    """Tests the Process function on a Google Chrome 59 History database."""
    plugin = chrome_history.GoogleChrome27HistoryPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['chrome_history', 'History-59.0.3071.86'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Check the page visit entry.
    expected_url = (
        'https://raw.githubusercontent.com/dfirlabs/chrome-specimens/master/'
        'generate-specimens.sh')

    expected_event_values = {
        'data_type': 'chrome:history:page_visited',
        'last_visited_time': '2018-01-21T14:08:52.037692+00:00',
        'title': '',
        'typed_count': 0,
        'visit_count': 1,
        'url': expected_url}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Check the file downloaded entry.
    expected_event_values = {
        'data_type': 'chrome:history:file_downloaded',
        'end_time': '2018-01-21T14:08:52.662377+00:00',
        'full_path': '/home/ubuntu/Downloads/plaso-20171231.1.win32.msi',
        'received_bytes': 3080192,
        'start_time': '2018-01-21T14:08:51.811123+00:00',
        'total_bytes': 3080192,
        'url': (
            'https://raw.githubusercontent.com/log2timeline/l2tbinaries/master/'
            'win32/plaso-20171231.1.win32.msi')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

  def testProcess59ExtraColumn(self):
    """Tests the Process function on a Google Chrome 59 History database.

    The History database has been modified to have an extra column.
    """
    plugin = chrome_history.GoogleChrome27HistoryPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['chrome_history', 'History-59_added-fake-column'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Check the page visit entry.
    expected_url = (
        'https://raw.githubusercontent.com/dfirlabs/chrome-specimens/master/'
        'generate-specimens.sh')

    expected_event_values = {
        'data_type': 'chrome:history:page_visited',
        'last_visited_time': '2018-01-21T14:08:52.037692+00:00',
        'title': '',
        'typed_count': 0,
        'visit_count': 1,
        'url': expected_url}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Check the file downloaded entry.
    expected_event_values = {
        'data_type': 'chrome:history:file_downloaded',
        'end_time': '2018-01-21T14:08:52.662377+00:00',
        'full_path': '/home/ubuntu/Downloads/plaso-20171231.1.win32.msi',
        'received_bytes': 3080192,
        'start_time': '2018-01-21T14:08:51.811123+00:00',
        'total_bytes': 3080192,
        'url': (
            'https://raw.githubusercontent.com/log2timeline/l2tbinaries/master/'
            'win32/plaso-20171231.1.win32.msi')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Container Runtime Interface (CRI) log text plugin."""

import unittest

from plaso.parsers.text_plugins import cri

from tests.parsers.text_plugins import test_lib


class CRILogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the CRI log text parser plugin."""

  def testProcess(self):
    """Tests for the CheckRequiredFormat method."""
    plugin = cri.CRITextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['cri.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 17)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'cri:container:log:entry',
        'event_datetime': '2016-10-06T00:17:09.669794202+00:00',
        'body': ' log content 1',
        'stream': 'stdout',
        'tag': 'P'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'cri:container:log:entry',
        'event_datetime': '2016-10-06T00:17:09.669794203+00:00',
        'body': ' log content 2',
        'stream': 'stderr',
        'tag': 'F'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'data_type': 'cri:container:log:entry',
        'event_datetime': '2024-04-16T06:25:29.095207860+00:00',
        'body': (
            ' 10.0.2.1:39914 - - [Tue, 16 Apr 2024 06:25:29 UTC] '
            '"GET /readiness HTTP/1.1" kube-probe/1.27'),
        'stream': 'stdout',
        'tag': 'F'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 14)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the multi-processing worker process."""

import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.containers import sessions
from plaso.containers import tasks
from plaso.engine import configurations
from plaso.engine import worker
from plaso.lib import definitions
from plaso.multi_process import extraction_process
from plaso.multi_process import plaso_queue
from plaso.multi_process import zeromq_queue

from tests import test_lib as shared_test_lib
from tests.multi_process import test_lib


class TestEventExtractionWorker(worker.EventExtractionWorker):
  """Event extraction worker for testing."""

  # pylint: disable=unused-argument
  def ProcessPathSpec(
      self, parser_mediator, path_spec, excluded_find_specs=None):
    """Processes a path specification.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      path_spec (dfvfs.PathSpec): path specification.
      excluded_find_specs (Optional[list[dfvfs.FindSpec]]): find specifications
         that are excluded from processing.
    """
    return


class WorkerProcessTest(test_lib.MultiProcessingTestCase):
  """Tests the multi-processing worker process."""

  # pylint: disable=protected-access

  _QUEUE_TIMEOUT = 5

  def testInitialization(self):
    """Tests the initialization."""
    with shared_test_lib.TempDirectory() as temp_directory:
      configuration = configurations.ProcessingConfiguration()
      configuration.task_storage_path = temp_directory

      test_process = extraction_process.ExtractionWorkerProcess(
          None, configuration, [], [], None, name='TestWorker')
      self.assertIsNotNone(test_process)

  def testGetStatus(self):
    """Tests the _GetStatus function."""
    with shared_test_lib.TempDirectory() as temp_directory:
      configuration = configurations.ProcessingConfiguration()
      configuration.task_storage_path = temp_directory

      test_process = extraction_process.ExtractionWorkerProcess(
          None, configuration, [], [], None, name='TestWorker')
      status_attributes = test_process._GetStatus()

      self.assertIsNotNone(status_attributes)
      self.assertEqual(status_attributes['identifier'], 'TestWorker')
      self.assertEqual(status_attributes['last_activity_timestamp'], 0.0)

      task_storage_writer = self._CreateStorageWriter()
      test_process._parser_mediator = self._CreateParserMediator(
          task_storage_writer)
      status_attributes = test_process._GetStatus()

      self.assertIsNotNone(status_attributes)
      self.assertEqual(status_attributes['identifier'], 'TestWorker')
      self.assertEqual(status_attributes['last_activity_timestamp'], 0.0)

  def testMain(self):
    """Tests the _Main function."""
    output_task_queue = zeromq_queue.ZeroMQBufferedReplyBindQueue(
        delay_open=True, linger_seconds=0, maximum_items=1,
        name='test output task queue', timeout_seconds=self._QUEUE_TIMEOUT)
    output_task_queue.Open()

    input_task_queue = zeromq_queue.ZeroMQRequestConnectQueue(
        delay_open=True, linger_seconds=0, name='test input task queue',
        port=output_task_queue.port, timeout_seconds=self._QUEUE_TIMEOUT)

    with shared_test_lib.TempDirectory() as temp_directory:
      configuration = configurations.ProcessingConfiguration()
      configuration.task_storage_path = temp_directory

      test_process = extraction_process.ExtractionWorkerProcess(
          input_task_queue, configuration, [], [], None, name='TestWorker')

      test_process.start()

      output_task_queue.PushItem(plaso_queue.QueueAbort(), block=False)
      output_task_queue.Close(abort=True)

  def testProcessPathSpec(self):
    """Tests the _ProcessPathSpec function."""
    test_file_path = self._GetTestFilePath(['testdir', 'filter_1.txt'])
    self._SkipIfPathNotExists(test_file_path)

    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)

    with shared_test_lib.TempDirectory() as temp_directory:
      configuration = configurations.ProcessingConfiguration()
      configuration.task_storage_path = temp_directory

      test_process = extraction_process.ExtractionWorkerProcess(
          None, configuration, [], [], None, name='TestWorker')

      task_storage_writer = self._CreateStorageWriter()
      parser_mediator = self._CreateParserMediator(task_storage_writer)

      extraction_worker = TestEventExtractionWorker()
      test_process._ProcessPathSpec(
          extraction_worker, parser_mediator, path_spec)
      self.assertEqual(parser_mediator._number_of_extraction_warnings, 0)

      test_process._ProcessPathSpec(None, parser_mediator, path_spec)
      self.assertEqual(parser_mediator._number_of_extraction_warnings, 1)

  def testProcessTask(self):
    """Tests the _ProcessTask function."""
    session = sessions.Session()
    with shared_test_lib.TempDirectory() as temp_directory:
      configuration = configurations.ProcessingConfiguration()
      configuration.task_storage_path = temp_directory
      configuration.task_storage_format = definitions.STORAGE_FORMAT_SQLITE

      test_process = extraction_process.ExtractionWorkerProcess(
          None, configuration, [], [], None, name='TestWorker')
      test_process._extraction_worker = TestEventExtractionWorker()

      task_storage_writer = self._CreateStorageWriter()
      test_process._parser_mediator = self._CreateParserMediator(
          task_storage_writer)

      task = tasks.Task(session_identifier=session.identifier)
      test_process._ProcessTask(task)

  def testStartAndStopProfiling(self):
    """Tests the _StartProfiling and _StopProfiling functions."""
    with shared_test_lib.TempDirectory() as temp_directory:
      configuration = configurations.ProcessingConfiguration()
      configuration.profiling.directory = temp_directory
      configuration.profiling.profilers = set([
          'memory', 'parsers', 'processing', 'serializers', 'storage',
          'task_queue'])
      configuration.task_storage_path = temp_directory

      test_process = extraction_process.ExtractionWorkerProcess(
          None, configuration, [], [], None, name='TestWorker')
      test_process._extraction_worker = TestEventExtractionWorker()

      test_process._StartProfiling(None)

      test_process._StartProfiling(configuration.profiling)
      test_process._StopProfiling()

  def testSignalAbort(self):
    """Tests the SignalAbort function."""
    with shared_test_lib.TempDirectory() as temp_directory:
      configuration = configurations.ProcessingConfiguration()
      configuration.task_storage_path = temp_directory

      test_process = extraction_process.ExtractionWorkerProcess(
          None, configuration, [], [], None, name='TestWorker')
      test_process.SignalAbort()


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the knowledge base."""

import unittest

from plaso.containers import artifacts
from plaso.engine import knowledge_base

from tests import test_lib as shared_test_lib


class KnowledgeBaseTest(shared_test_lib.BaseTestCase):
  """Tests for the knowledge base."""

  # pylint: disable=protected-access

  def testCodepageProperty(self):
    """Tests the codepage property."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    self.assertEqual(knowledge_base_object.codepage, 'cp1252')

  def testOperatingSystemProperty(self):
    """Tests the operating_system property."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    operating_system = knowledge_base_object.GetValue('operating_system')
    self.assertIsNone(operating_system)

    knowledge_base_object.SetValue('operating_system', 'Windows')

    operating_system = knowledge_base_object.GetValue('operating_system')
    self.assertEqual(operating_system, 'Windows')

  def testTimezoneProperty(self):
    """Tests the timezone property."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    self.assertEqual(knowledge_base_object.timezone.zone, 'UTC')

  def testAddEnvironmentVariable(self):
    """Tests the AddEnvironmentVariable function."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='SystemRoot', value='C:\\Windows')

    knowledge_base_object.AddEnvironmentVariable(environment_variable)

    with self.assertRaises(KeyError):
      knowledge_base_object.AddEnvironmentVariable(environment_variable)

  def testGetEnvironmentVariable(self):
    """Tests the GetEnvironmentVariable functions."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='SystemRoot', value='C:\\Windows')
    knowledge_base_object.AddEnvironmentVariable(environment_variable)

    test_environment_variable = knowledge_base_object.GetEnvironmentVariable(
        'SystemRoot')
    self.assertIsNotNone(test_environment_variable)

    test_environment_variable = knowledge_base_object.GetEnvironmentVariable(
        'sYsTeMrOoT')
    self.assertIsNotNone(test_environment_variable)

    test_environment_variable = knowledge_base_object.GetEnvironmentVariable(
        'Bogus')
    self.assertIsNone(test_environment_variable)

  def testGetEnvironmentVariables(self):
    """Tests the GetEnvironmentVariables function."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='SystemRoot', value='C:\\Windows')
    knowledge_base_object.AddEnvironmentVariable(environment_variable)

    environment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name='WinDir', value='C:\\Windows')
    knowledge_base_object.AddEnvironmentVariable(environment_variable)

    environment_variables = knowledge_base_object.GetEnvironmentVariables()
    self.assertEqual(len(environment_variables), 2)

  def testGetHostname(self):
    """Tests the GetHostname function."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    hostname = knowledge_base_object.GetHostname()
    self.assertEqual(hostname, '')

  def testGetSetValue(self):
    """Tests the Get and SetValue functions."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    expected_value = 'test value'
    knowledge_base_object.SetValue('Test', expected_value)

    value = knowledge_base_object.GetValue('Test')
    self.assertEqual(value, expected_value)

    value = knowledge_base_object.GetValue('tEsT')
    self.assertEqual(value, expected_value)

    value = knowledge_base_object.GetValue('Bogus')
    self.assertIsNone(value)

  def testReadSystemConfigurationArtifact(self):
    """Tests the ReadSystemConfigurationArtifact function."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    system_configuration = artifacts.SystemConfigurationArtifact()
    system_configuration.hostname = artifacts.HostnameArtifact(
        name='myhost.mydomain')

    knowledge_base_object.ReadSystemConfigurationArtifact(system_configuration)

    hostname = knowledge_base_object.GetHostname()
    self.assertEqual(hostname, 'myhost.mydomain')

  def testSetActiveSession(self):
    """Tests the SetActiveSession function."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    knowledge_base_object.SetActiveSession('ddda05bedf324cbd99fa8c24b8a0037a')
    self.assertEqual(
        knowledge_base_object._active_session,
        'ddda05bedf324cbd99fa8c24b8a0037a')

    knowledge_base_object.SetActiveSession(
        knowledge_base_object._DEFAULT_ACTIVE_SESSION)
    self.assertEqual(
        knowledge_base_object._active_session,
        knowledge_base_object._DEFAULT_ACTIVE_SESSION)

  def testSetCodepage(self):
    """Tests the SetCodepage function."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    knowledge_base_object.SetCodepage('cp1252')

    with self.assertRaises(ValueError):
      knowledge_base_object.SetCodepage('bogus')

  def testSetHostname(self):
    """Tests the SetHostname function."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    hostname_artifact = artifacts.HostnameArtifact(name='myhost.mydomain')
    knowledge_base_object.SetHostname(hostname_artifact)

  def testSetTimeZone(self):
    """Tests the SetTimeZone function."""
    knowledge_base_object = knowledge_base.KnowledgeBase()

    knowledge_base_object.SetTimeZone('Europe/Zurich')
    self.assertEqual(knowledge_base_object._time_zone.zone, 'Europe/Zurich')

    with self.assertRaises(ValueError):
      knowledge_base_object.SetTimeZone('Bogus')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the OpenSearchTimesketch output module."""

import unittest

from unittest.mock import MagicMock

from dfvfs.path import fake_path_spec

from plaso.lib import definitions
from plaso.output import opensearch_ts
from plaso.output import shared_opensearch

from tests.output import test_lib
from tests.containers import test_lib as containers_test_lib


class TestOpenSearchTimesketchOutputModule(
    opensearch_ts.OpenSearchTimesketchOutputModule):
  """OpenSearchsearch output module for testing."""

  def _Connect(self):
    """Connects to an OpenSearch server."""
    self._client = MagicMock()


class OpenSearchTimesketchOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests for the OpenSearchTimesketch output module."""

  # pylint: disable=protected-access

  _TEST_EVENTS = [
      {'data_type': 'syslog:line',
       'filename': 'log/syslog.1',
       'hostname': 'ubuntu',
       'my_number': 123,
       'some_additional_foo': True,
       'path_spec': fake_path_spec.FakePathSpec(
           location='log/syslog.1'),
       'text': (
           'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session\n '
           'closed for user root)'),
       'timestamp': '2012-06-27 18:17:01+00:00',
       'timestamp_desc': definitions.TIME_DESCRIPTION_WRITTEN}]

  def testFlushEvents(self):
    """Tests the _FlushEvents function.

    Raises:
      SkipTest: if opensearch-py is missing.
    """
    if shared_opensearch.opensearchpy is None:
      raise unittest.SkipTest('missing opensearch-py')

    output_mediator = self._CreateOutputMediator()

    formatters_directory_path = self._GetDataFilePath(['formatters'])
    output_mediator.ReadMessageFormattersFromDirectory(
        formatters_directory_path)

    output_module = TestOpenSearchTimesketchOutputModule()

    output_module._Connect()
    output_module._CreateIndexIfNotExists('test', {})

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    # TODO: add test for event_tag.
    field_values = output_module.GetFieldValues(
        output_mediator, event, event_data, event_data_stream, None)

    output_module.WriteFieldValues(output_mediator, field_values)

    self.assertEqual(len(output_module._event_documents), 2)
    self.assertEqual(output_module._number_of_buffered_events, 1)

    output_module._FlushEvents()

    self.assertEqual(len(output_module._event_documents), 0)
    self.assertEqual(output_module._number_of_buffered_events, 0)

  def testWriteFieldValues(self):
    """Tests the WriteFieldValues function.

    Raises:
      SkipTest: if opensearch-py is missing.
    """
    if shared_opensearch.opensearchpy is None:
      raise unittest.SkipTest('missing opensearch-py')

    output_mediator = self._CreateOutputMediator()

    formatters_directory_path = self._GetDataFilePath(['formatters'])
    output_mediator.ReadMessageFormattersFromDirectory(
        formatters_directory_path)

    output_module = TestOpenSearchTimesketchOutputModule()

    output_module._Connect()
    output_module._CreateIndexIfNotExists('test', {})

    self.assertEqual(len(output_module._event_documents), 0)
    self.assertEqual(output_module._number_of_buffered_events, 0)

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    # TODO: add test for event_tag.
    field_values = output_module.GetFieldValues(
        output_mediator, event, event_data, event_data_stream, None)

    output_module.WriteFieldValues(output_mediator, field_values)

    self.assertEqual(len(output_module._event_documents), 2)
    self.assertEqual(output_module._number_of_buffered_events, 1)

  def testWriteHeader(self):
    """Tests the WriteHeader function.

    Raises:
      SkipTest: if opensearch-py is missing.
    """
    if shared_opensearch.opensearchpy is None:
      raise unittest.SkipTest('missing opensearch-py')

    output_mediator = self._CreateOutputMediator()
    output_module = TestOpenSearchTimesketchOutputModule()

    self.assertIsNone(output_module._client)

    output_module.WriteHeader(output_mediator)

    self.assertIsNotNone(output_module._client)


if __name__ == '__main__':
  unittest.main()
